; <--- include all MeOS stuff --->
include "MACROS.INC"

include 'proc32.inc'

struc MEM_INFO
{ .pages_count	   dd ?
  .pages_free	   dd ?
  .page_faults	   dd ?
  .heap_size	   dd ?
  .heap_free	   dd ?
  .heap_blocks	   dd ?
  .free_blocks	   dd ?
  .largest_free    dd ?
  .largest_used    dd ?
}
MEM_INFO_SIZE equ 9*4

FPU_IM	    equ  0x0001
FPU_DM	    equ  0x0002
FPU_ZM	    equ  0x0004
FPU_OM	    equ  0x0008
FPU_UM	    equ  0x0010
FPU_PM	    equ  0x0020


; <--- start of MenuetOS application --->
MEOS_APP_START

; <--- start of code --->
CODE
	   call draw_window	 ; at first create and draw the window

wait_event:			 ; main cycle
	   mov eax, 10
	   int 0x40

	   cmp eax, 1		 ;   if event == 1
	   je  redraw		 ;     jump to redraw handler
	   cmp eax, 2		 ;   else if event == 2
	   je  key		 ;     jump to key handler
	   cmp eax, 3		 ;   else if event == 3
	   je  button		 ;     jump to button handler

	   jmp wait_event	 ;   else return to the start of main cycle
redraw: 			 ; redraw event handler
	   call    draw_window
	   jmp	   wait_event

key:
	   call GetKey			  ; key event handler

	   jmp	   wait_event

button: 			 ; button event handler
	   call GetButton
	   cmp	   ah, 1
	   jne	   wait_event	   ;   return if button id != 1

	   or	   eax, -1	   ;   exit application
	   int	   0x40


draw_window:
	   call BeginRedraw

	   stdcall DefineWindow, (100*65536+300),\
				 (100*65536+160),\
				 0x03ffffff,\
				 dword 0, dword 0

	   stdcall DrawText, (8*65536+8),\
			      0x10ffffff,\
			      header, header.size

	   stdcall GetMemInfo, mem_info

	   stdcall DrawText, (12*65536+28),\
			      0x10000000,\
			      msg_pages_count,\
			      msg_pages_count.size

	   mov eax, [mem_info.pages_count]
	   shl eax, 2
	   stdcall DrawNum, (160*65536+28),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+44),\
			      0x10000000,\
			      msg_pages_free,\
			      msg_pages_free.size

	   mov eax, [mem_info.pages_free]
	   shl eax, 2
	   stdcall DrawNum, (160*65536+44),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+60),\
			      0x10000000,\
			      msg_pages_fault,\
			      msg_pages_fault.size

	   mov eax, [mem_info.page_faults]
	   stdcall DrawNum, (160*65536+60),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+76),\
			      0x10000000,\
			      msg_heap_size,\
			      msg_heap_size.size

	   mov eax, [mem_info.heap_size]
	   shr eax, 10	    ;heap in Kbytes
	   stdcall DrawNum, (160*65536+76),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+92),\
			      0x10000000,\
			      msg_heap_free,\
			      msg_heap_free.size

	   mov eax, [mem_info.heap_free]
	   shr eax, 10	    ;free heap in Kbytes
	   stdcall DrawNum, (160*65536+92),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+108),\
			      0x10000000,\
			      msg_heap_blocks,\
			      msg_heap_blocks.size

	   mov eax, [mem_info.heap_blocks]
	   stdcall DrawNum, (160*65536+108),0x10000000,\
			    eax, 0x00080000

	   stdcall DrawText, (12*65536+124),\
			      0x10000000,\
			      msg_free_blocks,\
			      msg_free_blocks.size

	   mov eax, [mem_info.free_blocks]
	   stdcall DrawNum, (160*65536+124),0x10000000,\
			    eax, 0x00080000

	   call EndRedraw
	   ret

proc GetKey
	   mov eax, 2
	   int 0x40
	   ret
endp

proc GetButton
	   mov	   eax, 17     ;   get button identifier
	   int	   0x40
	   ret
endp

proc BeginRedraw
	   mov	   eax, 12     ; start drawing
	   mov	   ebx, 1
	   int	   0x40
	   ret
endp

proc EndRedraw
	   mov eax, 12		      ; finish drawing
	   mov ebx, 2
	   int 0x40
	   ret
endp

proc DefineWindow
	   xor eax, eax        ; create and draw the window
	   mov ebx, [esp+4]    ; (window_cx)*65536+(window_sx)
	   mov ecx, [esp+8]    ; (window_cy)*65536+(window_sy)
	   mov edx, [esp+12]   ; work area color & window type 3
	   mov esi, [esp+16]   ; grab color (not used)
	   mov edi, [esp+20]   ; frame color (not used)
	   int 0x40
	   ret 5*4
endp

proc DrawText
	   mov eax, 4	       ;  window header
	   mov ebx, [esp+4]    ;  coordinates
	   mov ecx, [esp+8]    ;  color & font N1
	   mov edx, [esp+12]   ;  address of text
	   mov esi, [esp+16]   ;  length of text
	   int 0x40
	   ret 4*4
endp

proc DrawNum
	   mov eax, 47	       ;
	   mov edx, [esp+4]    ; coordinates
	   mov esi, [esp+8]    ; color & font N1
	   mov ecx, [esp+12]   ; value
	   mov ebx, [esp+16]   ; flags
	   int 0x40
	   ret 4*4
endp

proc GetMemInfo stdcall, data_ptr:dword
	   mov eax, 18
	   mov ebx, 20
	   mov ecx, [data_ptr]
	   int 0x40
	   ret
endp

; <--- initialised data --->
DATA
  sz header, "Memory info"
  sz msg_pages_count, "TOTAL RAM Kb"
  sz msg_pages_free, "FREE RAM Kb"
  sz msg_pages_fault, "PAGE FAULTS"
  sz msg_heap_size,"HEAP SIZE  Kb"
  sz msg_heap_free,"HEAP FREE  Kb"
  sz msg_heap_blocks,"HEAP BLOCKS"
  sz msg_free_blocks,"FREE BLOCKS"

; <--- uninitialised data --->
UDATA

mem_info   MEM_INFO

MEOS_APP_END
; <--- end of MenuetOS application --->
